"""
The cheml.search module includes (please click on links adjacent to function names for more information):
    - GA_DEAP: :func:`~cheml.search.GA_DEAP`
"""

from .genetic_algorithm import GA_DEAP

__all__ = [
    'GA_DEAP',
]
