"""
The cheml.nn.keras module includes (please click on links adjacent to function names for more information):
    - MLP: :func:`~cheml.nn.keras.MLP`
    - MLP_sklearn: :func:`~cheml.nn.keras.MLP_sklearn`
"""


from .mlp import MLP
from .mlp import MLP_sklearn




__all__ = [
    'MLP',
    'MLP_sklearn'
    ]